@@ -46,7 +46,7 @@ import (
 	"github.com/pkg/errors"
 	"github.com/spf13/cobra"
 	appsv1 "k8s.io/api/apps/v1"
-	"k8s.io/api/batch/v1beta1"
+	batchv1 "k8s.io/api/batch/v1"
 	coordination "k8s.io/api/coordination/v1"
 	corev1 "k8s.io/api/core/v1"
 	rbacv1 "k8s.io/api/rbac/v1"
@@ -71,6 +71,7 @@ import (
 
 	"github.com/apache/camel-k/e2e/support/util"
 	v1 "github.com/apache/camel-k/pkg/apis/camel/v1"
+	traitv1 "github.com/apache/camel-k/pkg/apis/camel/v1/trait"
 	"github.com/apache/camel-k/pkg/apis/camel/v1alpha1"
 	"github.com/apache/camel-k/pkg/client"
 	"github.com/apache/camel-k/pkg/cmd"
@@ -91,11 +92,35 @@ const kubeConfigEnvVar = "KUBECONFIG"
 var TestTimeoutShort = 1 * time.Minute
 var TestTimeoutMedium = 5 * time.Minute
 var TestTimeoutLong = 10 * time.Minute
+
+// TestTimeoutVeryLong should be used only for testing native builds.
+var TestTimeoutVeryLong = 30 * time.Minute
+
 var NoOlmOperatorImage string
 
 var TestContext context.Context
 var testClient client.Client
 
+var testLocus *testing.T
+
+func setTestLocus(t *testing.T) {
+	testLocus = t
+}
+
+//
+// Only panic the test if absolutely necessary and there is
+// no test locus. In most cases, the test should fail gracefully
+// using the test locus to error out and fail now.
+//
+func failTest(err error) {
+	if testLocus != nil {
+		testLocus.Error(err)
+		testLocus.FailNow()
+	} else {
+		panic(err)
+	}
+}
+
 func TestClient() client.Client {
 	if testClient != nil {
 		return testClient
@@ -103,7 +128,7 @@ func TestClient() client.Client {
 	var err error
 	testClient, err = NewTestClient()
 	if err != nil {
-		panic(err)
+		failTest(err)
 	}
 	return testClient
 }
@@ -112,7 +137,7 @@ func SyncClient() client.Client {
 	var err error
 	testClient, err = NewTestClient()
 	if err != nil {
-		panic(err)
+		failTest(err)
 	}
 	return testClient
 }
@@ -197,6 +222,97 @@ func Kamel(args ...string) *cobra.Command {
 	return KamelWithContext(TestContext, args...)
 }
 
+func KamelInstall(namespace string, args ...string) *cobra.Command {
+	return KamelInstallWithID(platform.DefaultPlatformName, namespace, args...)
+}
+
+func KamelInstallWithID(operatorID string, namespace string, args ...string) *cobra.Command {
+	return KamelInstallWithContext(TestContext, operatorID, namespace, args...)
+}
+
+func KamelInstallWithContext(ctx context.Context, operatorID string, namespace string, args ...string) *cobra.Command {
+	var installArgs []string
+
+	globalTest := os.Getenv("CAMEL_K_FORCE_GLOBAL_TEST") == "true"
+	if globalTest {
+		fmt.Printf("Executing as global test\n")
+
+		if err := verifyGlobalOperator(); err != nil {
+			failTest(err)
+		}
+
+		// Have a global operator pod watching all namespaces
+		// so ensure an integration platform is installed in target namespace
+		installArgs = []string{"install", "--skip-operator-setup", "-n", namespace}
+	} else {
+		// NOT global so proceed with local namespaced kamel install using the operator id
+		installArgs = []string{"install", "-n", namespace, "--operator-id", operatorID}
+	}
+
+	installArgs = append(installArgs, args...)
+	return KamelWithContext(ctx, installArgs...)
+}
+
+func KamelRun(namespace string, args ...string) *cobra.Command {
+	return KamelRunWithID(platform.DefaultPlatformName, namespace, args...)
+}
+
+func KamelRunWithID(operatorID string, namespace string, args ...string) *cobra.Command {
+	return KamelRunWithContext(TestContext, operatorID, namespace, args...)
+}
+
+func KamelRunWithContext(ctx context.Context, operatorID string, namespace string, args ...string) *cobra.Command {
+	return KamelCommandWithContext(ctx, "run", operatorID, namespace, args...)
+}
+
+func KamelBind(namespace string, args ...string) *cobra.Command {
+	return KamelBindWithID(platform.DefaultPlatformName, namespace, args...)
+}
+
+func KamelBindWithID(operatorID string, namespace string, args ...string) *cobra.Command {
+	return KamelBindWithContext(TestContext, operatorID, namespace, args...)
+}
+
+func KamelBindWithContext(ctx context.Context, operatorID string, namespace string, args ...string) *cobra.Command {
+	return KamelCommandWithContext(ctx, "bind", operatorID, namespace, args...)
+}
+
+func KamelCommandWithContext(ctx context.Context, command string, operatorID string, namespace string, args ...string) *cobra.Command {
+	var cmdArgs []string
+
+	globalTest := os.Getenv("CAMEL_K_FORCE_GLOBAL_TEST") == "true"
+	if globalTest {
+		fmt.Printf("Running as globally managed resource\n")
+
+		if err := verifyGlobalOperator(); err != nil {
+			failTest(err)
+		}
+
+		// Have a global operator reconciling the integration
+		cmdArgs = []string{command, "-n", namespace}
+	} else {
+		// NOT global so proceed with local namespaced operator reconciling the integration
+		cmdArgs = []string{command, "-n", namespace, "--operator-id", operatorID}
+	}
+
+	cmdArgs = append(cmdArgs, args...)
+	return KamelWithContext(ctx, cmdArgs...)
+}
+
+func verifyGlobalOperator() error {
+	opns := os.Getenv("CAMEL_K_GLOBAL_OPERATOR_NS")
+	if opns == "" {
+		return errors.New("No operator namespace defined in CAMEL_K_GLOBAL_OPERATOR_NS")
+	}
+
+	oppod := OperatorPod(opns)()
+	if oppod == nil {
+		return fmt.Errorf("No operator pod detected in namespace %s. Operator install is a pre-requisite of the test", opns)
+	}
+
+	return nil
+}
+
 func KamelWithContext(ctx context.Context, args ...string) *cobra.Command {
 	var c *cobra.Command
 	var err error
@@ -208,7 +324,7 @@ func KamelWithContext(ctx context.Context, args ...string) *cobra.Command {
 	kamelBin := os.Getenv("KAMEL_BIN")
 	if kamelBin != "" {
 		if _, e := os.Stat(kamelBin); e != nil && os.IsNotExist(e) {
-			panic(e)
+			failTest(e)
 		}
 		fmt.Printf("Using external kamel binary on path %s\n", kamelBin)
 		c = &cobra.Command{
@@ -218,7 +334,7 @@ func KamelWithContext(ctx context.Context, args ...string) *cobra.Command {
 				var stdout io.Reader
 				stdout, err = externalBin.StdoutPipe()
 				if err != nil {
-					panic(err)
+					failTest(err)
 				}
 				err := externalBin.Start()
 				if err != nil {
@@ -240,7 +356,7 @@ func KamelWithContext(ctx context.Context, args ...string) *cobra.Command {
 		c, args, err = cmd.NewKamelWithModelineCommand(ctx, append([]string{"kamel"}, args...))
 	}
 	if err != nil {
-		panic(err)
+		failTest(err)
 	}
 	for _, hook := range KamelHooks {
 		args = hook(args)
@@ -260,15 +376,15 @@ func MakeWithContext(ctx context.Context, rule string, args ...string) *exec.Cmd
 
 	makeDir := os.Getenv("MAKE_DIR")
 	if makeDir == "" {
-		makeDir = "../../../install"
+		makeDir = "../../../../install"
 	} else {
 		fmt.Printf("Using alternative make directory on path %s\n", makeDir)
 	}
 
 	if fi, e := os.Stat(makeDir); e != nil && os.IsNotExist(e) {
-		panic(e)
+		failTest(e)
 	} else if !fi.Mode().IsDir() {
-		panic(e)
+		failTest(e)
 	}
 
 	args = append([]string{"-C", makeDir, rule}, args...)
@@ -288,7 +404,7 @@ func IntegrationLogs(ns, name string) func() string {
 		}
 
 		options := corev1.PodLogOptions{
-			TailLines: pointer.Int64Ptr(100),
+			TailLines: pointer.Int64(100),
 		}
 
 		if len(pod.Spec.Containers) > 1 {
@@ -413,7 +529,7 @@ func IntegrationPods(ns string, name string) func() []corev1.Pod {
 				v1.IntegrationLabel: name,
 			})
 		if err != nil {
-			panic(err)
+			failTest(err)
 		}
 		return lst.Items
 	}
@@ -429,6 +545,26 @@ func IntegrationSpecReplicas(ns string, name string) func() *int32 {
 	}
 }
 
+func IntegrationGeneration(ns string, name string) func() *int64 {
+	return func() *int64 {
+		it := Integration(ns, name)()
+		if it == nil {
+			return nil
+		}
+		return &it.Generation
+	}
+}
+
+func IntegrationObservedGeneration(ns string, name string) func() *int64 {
+	return func() *int64 {
+		it := Integration(ns, name)()
+		if it == nil {
+			return nil
+		}
+		return &it.Status.ObservedGeneration
+	}
+}
+
 func IntegrationStatusReplicas(ns string, name string) func() *int32 {
 	return func() *int32 {
 		it := Integration(ns, name)()
@@ -439,6 +575,26 @@ func IntegrationStatusReplicas(ns string, name string) func() *int32 {
 	}
 }
 
+func IntegrationStatusImage(ns string, name string) func() string {
+	return func() string {
+		it := Integration(ns, name)()
+		if it == nil {
+			return ""
+		}
+		return it.Status.Image
+	}
+}
+
+func IntegrationAnnotations(ns string, name string) func() map[string]string {
+	return func() map[string]string {
+		it := Integration(ns, name)()
+		if it == nil {
+			return map[string]string{}
+		}
+		return it.Annotations
+	}
+}
+
 func IntegrationCondition(ns string, name string, conditionType v1.IntegrationConditionType) func() *v1.IntegrationCondition {
 	return func() *v1.IntegrationCondition {
 		it := Integration(ns, name)()
@@ -482,13 +638,15 @@ func AssignIntegrationToOperator(ns, name, operator string) error {
 	if it == nil {
 		return fmt.Errorf("cannot assign integration %q to operator: integration not found", name)
 	}
-	if it.Annotations == nil {
-		it.Annotations = make(map[string]string)
-	}
-	it.Annotations[v1.OperatorIDAnnotation] = operator
+
+	it.SetOperatorID(operator)
 	return TestClient().Update(TestContext, it)
 }
 
+func Annotations(object metav1.Object) map[string]string {
+	return object.GetAnnotations()
+}
+
 func Lease(ns string, name string) func() *coordination.Lease {
 	return func() *coordination.Lease {
 		lease := coordination.Lease{}
@@ -500,7 +658,7 @@ func Lease(ns string, name string) func() *coordination.Lease {
 		if err != nil && k8serrors.IsNotFound(err) {
 			return nil
 		} else if err != nil {
-			panic(err)
+			failTest(err)
 		}
 		return &lease
 	}
@@ -516,7 +674,7 @@ func Nodes() func() []corev1.Node {
 		}
 		err := TestClient().List(TestContext, nodes)
 		if err != nil {
-			panic(err)
+			failTest(err)
 		}
 		return nodes.Items
 	}
@@ -535,7 +693,7 @@ func Node(name string) func() *corev1.Node {
 		}
 		err := TestClient().Get(TestContext, ctrl.ObjectKeyFromObject(node), node)
 		if err != nil {
-			panic(err)
+			failTest(err)
 		}
 		return node
 	}
@@ -552,12 +710,50 @@ func Service(ns string, name string) func() *corev1.Service {
 		if err != nil && k8serrors.IsNotFound(err) {
 			return nil
 		} else if err != nil {
-			panic(err)
+			failTest(err)
 		}
 		return &svc
 	}
 }
 
+func ServiceType(ns string, name string) func() corev1.ServiceType {
+	return func() corev1.ServiceType {
+		svc := Service(ns, name)()
+		if svc == nil {
+			return ""
+		}
+		return svc.Spec.Type
+	}
+}
+
+//
+// Find the service in the given namespace with the given type
+//
+func ServicesByType(ns string, svcType corev1.ServiceType) func() []corev1.Service {
+	return func() []corev1.Service {
+		svcs := []corev1.Service{}
+
+		svcList, err := TestClient().CoreV1().Services(ns).List(TestContext, metav1.ListOptions{})
+		if err != nil && k8serrors.IsNotFound(err) {
+			return svcs
+		} else if err != nil {
+			failTest(err)
+		}
+
+		if len(svcList.Items) == 0 {
+			return svcs
+		}
+
+		for _, svc := range svcList.Items {
+			if svc.Spec.Type == svcType {
+				svcs = append(svcs, svc)
+			}
+		}
+
+		return svcs
+	}
+}
+
 func Route(ns string, name string) func() *routev1.Route {
 	return func() *routev1.Route {
 		route := routev1.Route{}
@@ -569,7 +765,7 @@ func Route(ns string, name string) func() *routev1.Route {
 		if err != nil && k8serrors.IsNotFound(err) {
 			return nil
 		} else if err != nil {
-			panic(err)
+			failTest(err)
 		}
 		return &route
 	}
@@ -585,12 +781,12 @@ func RouteStatus(ns string, name string) func() string {
 	}
 }
 
-func IntegrationCronJob(ns string, name string) func() *v1beta1.CronJob {
-	return func() *v1beta1.CronJob {
-		lst := v1beta1.CronJobList{
+func IntegrationCronJob(ns string, name string) func() *batchv1.CronJob {
+	return func() *batchv1.CronJob {
+		lst := batchv1.CronJobList{
 			TypeMeta: metav1.TypeMeta{
 				Kind:       "CronJob",
-				APIVersion: v1beta1.SchemeGroupVersion.String(),
+				APIVersion: batchv1.SchemeGroupVersion.String(),
 			},
 		}
 		err := TestClient().List(TestContext, &lst,
@@ -599,7 +795,7 @@ func IntegrationCronJob(ns string, name string) func() *v1beta1.CronJob {
 				"camel.apache.org/integration": name,
 			})
 		if err != nil {
-			panic(err)
+			failTest(err)
 		}
 		if len(lst.Items) == 0 {
 			return nil
@@ -616,7 +812,7 @@ func Integration(ns string, name string) func() *v1.Integration {
 			Name:      name,
 		}
 		if err := TestClient().Get(TestContext, key, &it); err != nil && !k8serrors.IsNotFound(err) {
-			panic(err)
+			failTest(err)
 		} else if err != nil && k8serrors.IsNotFound(err) {
 			return nil
 		}
@@ -681,7 +877,7 @@ func Kit(ns, name string) func() *v1.IntegrationKit {
 	return func() *v1.IntegrationKit {
 		kit := v1.NewIntegrationKit(ns, name)
 		if err := TestClient().Get(TestContext, ctrl.ObjectKeyFromObject(kit), kit); err != nil && !k8serrors.IsNotFound(err) {
-			panic(err)
+			failTest(err)
 		} else if err != nil && k8serrors.IsNotFound(err) {
 			return nil
 		}
@@ -739,7 +935,7 @@ func KameletBinding(ns string, name string) func() *v1alpha1.KameletBinding {
 			Name:      name,
 		}
 		if err := TestClient().Get(TestContext, key, &klb); err != nil && !k8serrors.IsNotFound(err) {
-			panic(err)
+			failTest(err)
 		} else if err != nil && k8serrors.IsNotFound(err) {
 			return nil
 		}
@@ -837,10 +1033,8 @@ func AssignKameletBindingToOperator(ns, name, operator string) error {
 	if klb == nil {
 		return fmt.Errorf("cannot assign kamelet binding %q to operator: kamelet binding not found", name)
 	}
-	if klb.Annotations == nil {
-		klb.Annotations = make(map[string]string)
-	}
-	klb.Annotations[v1.OperatorIDAnnotation] = operator
+
+	klb.SetOperatorID(operator)
 	return TestClient().Update(TestContext, klb)
 }
 
@@ -888,14 +1082,14 @@ func Kits(ns string, options ...interface{}) func() []v1.IntegrationKit {
 		case ctrl.ListOption:
 			listOptions = append(listOptions, o)
 		default:
-			panic(fmt.Errorf("unsupported kits option %q", o))
+			failTest(fmt.Errorf("unsupported kits option %q", o))
 		}
 	}
 
 	return func() []v1.IntegrationKit {
 		list := v1.NewIntegrationKitList()
 		if err := TestClient().List(TestContext, &list, listOptions...); err != nil {
-			panic(err)
+			failTest(err)
 		}
 
 		var kits []v1.IntegrationKit
@@ -975,13 +1169,13 @@ func AutogeneratedConfigmapsCount(ns string) func() int {
 				kubernetes.ConfigMapAutogenLabel: "true",
 			})
 		if err != nil {
-			panic(err)
+			failTest(err)
 		}
 		return len(lst.Items)
 	}
 }
 
-func NewPlainTextConfigmap(ns string, name string, data map[string]string) error {
+func CreatePlainTextConfigmap(ns string, name string, data map[string]string) error {
 	cm := corev1.ConfigMap{
 		TypeMeta: metav1.TypeMeta{
 			Kind:       "ConfigMap",
@@ -996,7 +1190,22 @@ func NewPlainTextConfigmap(ns string, name string, data map[string]string) error
 	return TestClient().Create(TestContext, &cm)
 }
 
-func NewBinaryConfigmap(ns string, name string, data map[string][]byte) error {
+func UpdatePlainTextConfigmap(ns string, name string, data map[string]string) error {
+	cm := corev1.ConfigMap{
+		TypeMeta: metav1.TypeMeta{
+			Kind:       "ConfigMap",
+			APIVersion: corev1.SchemeGroupVersion.String(),
+		},
+		ObjectMeta: metav1.ObjectMeta{
+			Namespace: ns,
+			Name:      name,
+		},
+		Data: data,
+	}
+	return TestClient().Update(TestContext, &cm)
+}
+
+func CreateBinaryConfigmap(ns string, name string, data map[string][]byte) error {
 	cm := corev1.ConfigMap{
 		TypeMeta: metav1.TypeMeta{
 			Kind:       "ConfigMap",
@@ -1011,7 +1220,7 @@ func NewBinaryConfigmap(ns string, name string, data map[string][]byte) error {
 	return TestClient().Create(TestContext, &cm)
 }
 
-func NewPlainTextSecret(ns string, name string, data map[string]string) error {
+func CreatePlainTextSecret(ns string, name string, data map[string]string) error {
 	sec := corev1.Secret{
 		TypeMeta: metav1.TypeMeta{
 			Kind:       "Secret",
@@ -1120,11 +1329,21 @@ func BuildPhase(ns, name string) func() v1.BuildPhase {
 	}
 }
 
+func HasPlatform(ns string) func() bool {
+	return func() bool {
+		lst := v1.NewIntegrationPlatformList()
+		if err := TestClient().List(TestContext, &lst, ctrl.InNamespace(ns)); err != nil {
+			return false
+		}
+		return len(lst.Items) > 0
+	}
+}
+
 func Platform(ns string) func() *v1.IntegrationPlatform {
 	return func() *v1.IntegrationPlatform {
 		lst := v1.NewIntegrationPlatformList()
 		if err := TestClient().List(TestContext, &lst, ctrl.InNamespace(ns)); err != nil {
-			panic(err)
+			failTest(err)
 		}
 		if len(lst.Items) == 0 {
 			return nil
@@ -1137,12 +1356,12 @@ func Platform(ns string) func() *v1.IntegrationPlatform {
 					continue
 				}
 				if pl != nil {
-					panic("multiple primary integration platforms found in namespace " + ns)
+					failTest(fmt.Errorf("multiple primary integration platforms found in namespace %q", ns))
 				}
 				pl = &p
 			}
 			if pl == nil {
-				panic(fmt.Sprintf("multiple integration platforms found in namespace %q but no one is primary", ns))
+				failTest(fmt.Errorf("multiple integration platforms found in namespace %q but no one is primary", ns))
 			}
 			return pl
 		}
@@ -1199,22 +1418,18 @@ func AssignPlatformToOperator(ns, operator string) error {
 	if pl == nil {
 		return errors.New("cannot assign platform to operator: no platform found")
 	}
-	if pl.Annotations == nil {
-		pl.Annotations = make(map[string]string)
-	}
-	pl.Annotations[v1.OperatorIDAnnotation] = operator
+
+	pl.SetOperatorID(operator)
 	return TestClient().Update(TestContext, pl)
 }
 
-func ConfigureSecondayPlatfromWith(ns string, customizer func(pl *v1.IntegrationPlatform)) error {
+func ConfigureSecondaryPlatformWith(ns string, customizer func(pl *v1.IntegrationPlatform)) error {
 	pl := Platform(ns)()
 	if pl == nil {
 		return errors.New("cannot find primary platform")
 	}
-	if pl.Annotations == nil {
-		pl.Annotations = make(map[string]string)
-	}
-	pl.Annotations[v1.SecondaryPlatformAnnotation] = "true"
+
+	v1.SetAnnotation(&pl.ObjectMeta, v1.SecondaryPlatformAnnotation, "true")
 	pl.ObjectMeta.ResourceVersion = ""
 	pl.Name = ""
 	pl.Status = v1.IntegrationPlatformStatus{}
@@ -1242,7 +1457,7 @@ func CRDs() func() []metav1.APIResource {
 			if err != nil && k8serrors.IsNotFound(err) {
 				return nil
 			} else if err != nil {
-				panic(err)
+				failTest(err)
 			}
 
 			for _, res := range lst.APIResources {
@@ -1266,7 +1481,7 @@ func ConsoleCLIDownload(name string) func() *consoleV1.ConsoleCLIDownload {
 	return func() *consoleV1.ConsoleCLIDownload {
 		cliDownload := consoleV1.ConsoleCLIDownload{}
 		if err := TestClient().Get(TestContext, ctrl.ObjectKey{Name: name}, &cliDownload); err != nil && !k8serrors.IsNotFound(err) {
-			panic(err)
+			failTest(err)
 		} else if err != nil && k8serrors.IsNotFound(err) {
 			return nil
 		}
@@ -1275,6 +1490,15 @@ func ConsoleCLIDownload(name string) func() *consoleV1.ConsoleCLIDownload {
 }
 
 func OperatorPod(ns string) func() *corev1.Pod {
+	namespace := ns
+
+	globalTest := os.Getenv("CAMEL_K_FORCE_GLOBAL_TEST") == "true"
+	opns := os.Getenv("CAMEL_K_GLOBAL_OPERATOR_NS")
+	if globalTest && len(opns) > 0 {
+		// Use the global operator pod instead of given namespace
+		namespace = opns
+	}
+
 	return func() *corev1.Pod {
 		lst := corev1.PodList{
 			TypeMeta: metav1.TypeMeta{
@@ -1283,12 +1507,12 @@ func OperatorPod(ns string) func() *corev1.Pod {
 			},
 		}
 		err := TestClient().List(TestContext, &lst,
-			ctrl.InNamespace(ns),
+			ctrl.InNamespace(namespace),
 			ctrl.MatchingLabels{
 				"camel.apache.org/component": "operator",
 			})
 		if err != nil {
-			panic(err)
+			failTest(err)
 		}
 		if len(lst.Items) == 0 {
 			return nil
@@ -1337,7 +1561,7 @@ func ClusterRole() func() []rbacv1.ClusterRole {
 				"app": "camel-k",
 			})
 		if err != nil {
-			panic(err)
+			failTest(err)
 		}
 		if len(lst.Items) == 0 {
 			return nil
@@ -1360,7 +1584,7 @@ func Role(ns string) func() []rbacv1.Role {
 				"app": "camel-k",
 			})
 		if err != nil {
-			panic(err)
+			failTest(err)
 		}
 		if len(lst.Items) == 0 {
 			return nil
@@ -1383,7 +1607,7 @@ func RoleBinding(ns string) func() *rbacv1.RoleBinding {
 				"app": "camel-k",
 			})
 		if err != nil {
-			panic(err)
+			failTest(err)
 		}
 		if len(lst.Items) == 0 {
 			return nil
@@ -1406,7 +1630,7 @@ func ServiceAccount(ns, name string) func() *corev1.ServiceAccount {
 				"app": "camel-k",
 			})
 		if err != nil {
-			panic(err)
+			failTest(err)
 		}
 		if len(lst.Items) == 0 {
 			return nil
@@ -1420,7 +1644,7 @@ func KameletList(ns string) func() []v1alpha1.Kamelet {
 		lst := v1alpha1.NewKameletList()
 		err := TestClient().List(TestContext, &lst, ctrl.InNamespace(ns))
 		if err != nil {
-			panic(err)
+			failTest(err)
 		}
 		return lst.Items
 	}
@@ -1434,7 +1658,7 @@ func Kamelet(name string, ns string) func() *v1alpha1.Kamelet {
 			Name:      name,
 		}
 		if err := TestClient().Get(TestContext, key, &it); err != nil && !k8serrors.IsNotFound(err) {
-			panic(err)
+			failTest(err)
 		} else if err != nil && k8serrors.IsNotFound(err) {
 			return nil
 		}
@@ -1472,7 +1696,7 @@ func CreateOperatorServiceAccount(ns string) error {
 func CreateOperatorRole(ns string) (err error) {
 	oc, err := openshift.IsOpenShift(TestClient())
 	if err != nil {
-		panic(err)
+		failTest(err)
 	}
 	customizer := install.IdentityResourceCustomizer
 	if oc {
@@ -1493,7 +1717,7 @@ func CreateOperatorRole(ns string) (err error) {
 func CreateOperatorRoleBinding(ns string) error {
 	oc, err := openshift.IsOpenShift(TestClient())
 	if err != nil {
-		panic(err)
+		failTest(err)
 	}
 	err = install.Resource(TestContext, TestClient(), ns, true, install.IdentityResourceCustomizer, "/rbac/operator-role-binding.yaml")
 	if err != nil {
@@ -1554,6 +1778,22 @@ func CreateKnativeChannel(ns string, name string) func() error {
 	}
 }
 
+func CreateKnativeBroker(ns string, name string) func() error {
+	return func() error {
+		broker := eventing.Broker{
+			TypeMeta: metav1.TypeMeta{
+				Kind:       "Broker",
+				APIVersion: eventing.SchemeGroupVersion.String(),
+			},
+			ObjectMeta: metav1.ObjectMeta{
+				Namespace: ns,
+				Name:      name,
+			},
+		}
+		return TestClient().Create(TestContext, &broker)
+	}
+}
+
 /*
 	Kamelets
 */
@@ -1624,14 +1864,15 @@ func BindKameletToWithErrorHandler(ns string, name string, annotations map[strin
 		if errorHandler != nil {
 			kb.Spec.ErrorHandler = asErrorHandlerSpec(errorHandler)
 		}
-		return kubernetes.ReplaceResource(TestContext, TestClient(), &kb)
+		_, err := kubernetes.ReplaceResource(TestContext, TestClient(), &kb)
+		return err
 	}
 }
 
 func asTemplate(source map[string]interface{}) *v1alpha1.Template {
 	bytes, err := json.Marshal(source)
 	if err != nil {
-		panic(err)
+		failTest(err)
 	}
 	return &v1alpha1.Template{
 		RawMessage: bytes,
@@ -1641,7 +1882,7 @@ func asTemplate(source map[string]interface{}) *v1alpha1.Template {
 func asErrorHandlerSpec(source map[string]interface{}) *v1alpha1.ErrorHandlerSpec {
 	bytes, err := json.Marshal(source)
 	if err != nil {
-		panic(err)
+		failTest(err)
 	}
 	return &v1alpha1.ErrorHandlerSpec{
 		RawMessage: bytes,
@@ -1651,19 +1892,20 @@ func asErrorHandlerSpec(source map[string]interface{}) *v1alpha1.ErrorHandlerSpe
 func asEndpointProperties(props map[string]string) *v1alpha1.EndpointProperties {
 	bytes, err := json.Marshal(props)
 	if err != nil {
-		panic(err)
+		failTest(err)
 	}
 	return &v1alpha1.EndpointProperties{
 		RawMessage: bytes,
 	}
 }
 
-func AsTraitConfiguration(props map[string]string) v1.TraitConfiguration {
+// nolint: staticcheck
+func AsTraitConfiguration(props map[string]string) *traitv1.Configuration {
 	bytes, err := json.Marshal(props)
 	if err != nil {
-		panic(err)
+		failTest(err)
 	}
-	return v1.TraitConfiguration{
+	return &traitv1.Configuration{
 		RawMessage: bytes,
 	}
 }
@@ -1691,19 +1933,21 @@ func NumPods(ns string) func() int {
 }
 
 func WithNewTestNamespace(t *testing.T, doRun func(string)) {
-	ns := NewTestNamespace(false)
-	defer DeleteTestNamespace(t, ns)
-	defer UserCleanup(t)
+	setTestLocus(t)
+	ns := newTestNamespace(false)
+	defer deleteTestNamespace(t, ns)
+	defer userCleanup(t)
 
-	InvokeUserTestCode(t, ns.GetName(), doRun)
+	invokeUserTestCode(t, ns.GetName(), doRun)
 }
 
 func WithGlobalOperatorNamespace(t *testing.T, test func(string)) {
+	setTestLocus(t)
 	ocp, err := openshift.IsOpenShift(TestClient())
 	assert.Nil(t, err)
 	if ocp {
 		// global operators are always installed in the openshift-operators namespace
-		InvokeUserTestCode(t, "openshift-operators", test)
+		invokeUserTestCode(t, "openshift-operators", test)
 	} else {
 		// create new namespace for the global operator
 		WithNewTestNamespace(t, test)
@@ -1711,15 +1955,16 @@ func WithGlobalOperatorNamespace(t *testing.T, test func(string)) {
 }
 
 func WithNewTestNamespaceWithKnativeBroker(t *testing.T, doRun func(string)) {
-	ns := NewTestNamespace(true)
-	defer DeleteTestNamespace(t, ns)
-	defer DeleteKnativeBroker(ns)
-	defer UserCleanup(t)
+	setTestLocus(t)
+	ns := newTestNamespace(true)
+	defer deleteTestNamespace(t, ns)
+	defer deleteKnativeBroker(ns)
+	defer userCleanup(t)
 
-	InvokeUserTestCode(t, ns.GetName(), doRun)
+	invokeUserTestCode(t, ns.GetName(), doRun)
 }
 
-func UserCleanup(t *testing.T) {
+func userCleanup(t *testing.T) {
 	userCmd := os.Getenv("KAMEL_TEST_CLEANUP")
 	if userCmd != "" {
 		fmt.Printf("Executing user cleanup command: %s\n", userCmd)
@@ -1735,20 +1980,33 @@ func UserCleanup(t *testing.T) {
 	}
 }
 
-func InvokeUserTestCode(t *testing.T, ns string, doRun func(string)) {
-	defer func() {
+func invokeUserTestCode(t *testing.T, ns string, doRun func(string)) {
+	globalTest := os.Getenv("CAMEL_K_FORCE_GLOBAL_TEST") == "true"
+
+	defer func(isGlobal bool) {
 		if t.Failed() {
 			if err := util.Dump(TestContext, TestClient(), ns, t); err != nil {
 				t.Logf("Error while dumping namespace %s: %v\n", ns, err)
 			}
 		}
-	}()
+
+		// Try to clean up namespace
+		if !isGlobal && HasPlatform(ns)() {
+			t.Logf("Clean up test namespace: %s", ns)
+
+			if err := Kamel("uninstall", "-n", ns, "--skip-crd", "--skip-cluster-roles").Execute(); err != nil {
+				t.Logf("Error while cleaning up namespace %s: %v\n", ns, err)
+			}
+
+			t.Logf("Successfully cleaned up test namespace: %s", ns)
+		}
+	}(globalTest)
 
 	gomega.RegisterTestingT(t)
 	doRun(ns)
 }
 
-func DeleteKnativeBroker(ns metav1.Object) {
+func deleteKnativeBroker(ns metav1.Object) {
 	nsRef := corev1.Namespace{
 		TypeMeta: metav1.TypeMeta{
 			APIVersion: v1.SchemeGroupVersion.String(),
@@ -1760,12 +2018,12 @@ func DeleteKnativeBroker(ns metav1.Object) {
 	}
 	nsKey := ctrl.ObjectKeyFromObject(&nsRef)
 	if err := TestClient().Get(TestContext, nsKey, &nsRef); err != nil {
-		panic(err)
+		failTest(err)
 	}
 
 	nsRef.SetLabels(make(map[string]string, 0))
 	if err := TestClient().Update(TestContext, &nsRef); err != nil {
-		panic(err)
+		failTest(err)
 	}
 	broker := eventing.Broker{
 		TypeMeta: metav1.TypeMeta{
@@ -1778,11 +2036,11 @@ func DeleteKnativeBroker(ns metav1.Object) {
 		},
 	}
 	if err := TestClient().Delete(TestContext, &broker); err != nil {
-		panic(err)
+		failTest(err)
 	}
 }
 
-func DeleteTestNamespace(t *testing.T, ns ctrl.Object) {
+func deleteTestNamespace(t *testing.T, ns ctrl.Object) {
 	value, saveNS := os.LookupEnv("CAMEL_K_TEST_SAVE_FAILED_TEST_NAMESPACE")
 	if t.Failed() && saveNS && value == "true" {
 		t.Logf("Warning: retaining failed test project %q", ns.GetName())
@@ -1792,7 +2050,7 @@ func DeleteTestNamespace(t *testing.T, ns ctrl.Object) {
 	var oc bool
 	var err error
 	if oc, err = openshift.IsOpenShift(TestClient()); err != nil {
-		panic(err)
+		failTest(err)
 	} else if oc {
 		prj := &projectv1.Project{
 			TypeMeta: metav1.TypeMeta{
@@ -1823,19 +2081,19 @@ func DeleteTestNamespace(t *testing.T, ns ctrl.Object) {
 	}
 }
 
-func NewTestNamespace(injectKnativeBroker bool) ctrl.Object {
+func newTestNamespace(injectKnativeBroker bool) ctrl.Object {
 	brokerLabel := "eventing.knative.dev/injection"
 	name := "test-" + uuid.New().String()
 	c := TestClient()
 
 	if oc, err := openshift.IsOpenShift(TestClient()); err != nil {
-		panic(err)
+		failTest(err)
 	} else if oc {
 		rest, err := apiutil.RESTClientForGVK(
 			schema.GroupVersionKind{Group: projectv1.GroupName, Version: projectv1.GroupVersion.Version}, false,
 			c.GetConfig(), serializer.NewCodecFactory(c.GetScheme()))
 		if err != nil {
-			panic(err)
+			failTest(err)
 		}
 		request := &projectv1.ProjectRequest{
 			TypeMeta: metav1.TypeMeta{
@@ -1858,20 +2116,20 @@ func NewTestNamespace(injectKnativeBroker bool) ctrl.Object {
 			Do(TestContext).
 			Into(project)
 		if err != nil {
-			panic(err)
+			failTest(err)
 		}
 		// workaround https://github.com/openshift/origin/issues/3819
 		if injectKnativeBroker {
 			// use Kubernetes API - https://access.redhat.com/solutions/2677921
 			if namespace, err := TestClient().CoreV1().Namespaces().Get(TestContext, name, metav1.GetOptions{}); err != nil {
-				panic(err)
+				failTest(err)
 			} else {
 				if _, ok := namespace.GetLabels()[brokerLabel]; !ok {
 					namespace.SetLabels(map[string]string{
 						brokerLabel: "enabled",
 					})
 					if err = TestClient().Update(TestContext, namespace); err != nil {
-						panic("Unable to label project with knative-eventing-injection. This operation needs update permission on the project.")
+						failTest(errors.New("Unable to label project with knative-eventing-injection. This operation needs update permission on the project."))
 					}
 				}
 			}
@@ -1893,10 +2151,12 @@ func NewTestNamespace(injectKnativeBroker bool) ctrl.Object {
 			})
 		}
 		if err := TestClient().Create(TestContext, namespace); err != nil {
-			panic(err)
+			failTest(err)
 		}
 		return namespace
 	}
+
+	return nil
 }
 
 func GetOutputString(command *cobra.Command) string {
