@@ -19,7 +19,6 @@ package integration
 
 import (
 	"context"
-	"encoding/json"
 	"errors"
 	"fmt"
 	"reflect"
@@ -27,9 +26,7 @@ import (
 
 	appsv1 "k8s.io/api/apps/v1"
 	batchv1 "k8s.io/api/batch/v1"
-	batchv1beta1 "k8s.io/api/batch/v1beta1"
 	corev1 "k8s.io/api/core/v1"
-	"k8s.io/apimachinery/pkg/api/equality"
 	k8serrors "k8s.io/apimachinery/pkg/api/errors"
 	"k8s.io/apimachinery/pkg/labels"
 	"k8s.io/apimachinery/pkg/selection"
@@ -71,6 +68,11 @@ func (action *monitorAction) Handle(ctx context.Context, integration *v1.Integra
 		return nil, fmt.Errorf("no kit set on integration %s", integration.Name)
 	}
 
+	kit, err := kubernetes.GetIntegrationKit(ctx, action.client, integration.Status.IntegrationKit.Name, integration.Status.IntegrationKit.Namespace)
+	if err != nil {
+		return nil, fmt.Errorf("unable to find integration kit %s/%s: %w", integration.Status.IntegrationKit.Namespace, integration.Status.IntegrationKit.Name, err)
+	}
+
 	// Check if the Integration requires a rebuild
 	hash, err := digest.ComputeForIntegration(integration)
 	if err != nil {
@@ -78,7 +80,13 @@ func (action *monitorAction) Handle(ctx context.Context, integration *v1.Integra
 	}
 
 	if hash != integration.Status.Digest {
-		action.L.Info("Integration needs a rebuild")
+		action.L.Info("Monitor: Integration needs a rebuild")
+
+		if v1.GetOperatorIDAnnotation(integration) != "" &&
+			(v1.GetOperatorIDAnnotation(integration) != v1.GetOperatorIDAnnotation(kit)) {
+			// Operator to reconcile the integration has changed. Reset integration kit so new operator can handle the kit reference
+			integration.SetIntegrationKit(nil)
+		}
 
 		integration.Initialize()
 		integration.Status.Digest = hash
@@ -86,11 +94,6 @@ func (action *monitorAction) Handle(ctx context.Context, integration *v1.Integra
 		return integration, nil
 	}
 
-	kit, err := kubernetes.GetIntegrationKit(ctx, action.client, integration.Status.IntegrationKit.Name, integration.Status.IntegrationKit.Namespace)
-	if err != nil {
-		return nil, fmt.Errorf("unable to find integration kit %s/%s: %w", integration.Status.IntegrationKit.Namespace, integration.Status.IntegrationKit.Name, err)
-	}
-
 	// Check if an IntegrationKit with higher priority is ready
 	priority, ok := kit.Labels[v1.IntegrationKitPriorityLabel]
 	if !ok {
@@ -164,85 +167,98 @@ func (action *monitorAction) Handle(ctx context.Context, integration *v1.Integra
 	return integration, nil
 }
 
-func (action *monitorAction) updateIntegrationPhaseAndReadyCondition(ctx context.Context, environment *trait.Environment, integration *v1.Integration, pendingPods []corev1.Pod, runningPods []corev1.Pod) error {
-	var controller ctrl.Object
-	var lastCompletedJob *batchv1.Job
-	var podSpec corev1.PodSpec
+type controller interface {
+	checkReadyCondition(ctx context.Context) (bool, error)
+	getPodSpec() corev1.PodSpec
+	updateReadyCondition(readyPods []corev1.Pod) bool
+}
 
+func (action *monitorAction) newController(env *trait.Environment, integration *v1.Integration) (controller, error) {
+	var controller controller
+	var obj ctrl.Object
 	switch {
 	case isConditionTrue(integration, v1.IntegrationConditionDeploymentAvailable):
-		controller = &appsv1.Deployment{}
+		obj = getUpdatedController(env, &appsv1.Deployment{})
+		deploy, ok := obj.(*appsv1.Deployment)
+		if !ok {
+			return nil, fmt.Errorf("type assertion failed: %v", obj)
+		}
+		controller = &deploymentController{
+			obj:         deploy,
+			integration: integration,
+		}
 	case isConditionTrue(integration, v1.IntegrationConditionKnativeServiceAvailable):
-		controller = &servingv1.Service{}
+		obj = getUpdatedController(env, &servingv1.Service{})
+		svc, ok := obj.(*servingv1.Service)
+		if !ok {
+			return nil, fmt.Errorf("type assertion failed: %v", obj)
+		}
+		controller = &knativeServiceController{
+			obj:         svc,
+			integration: integration,
+		}
 	case isConditionTrue(integration, v1.IntegrationConditionCronJobAvailable):
-		controller = &batchv1beta1.CronJob{}
+		obj = getUpdatedController(env, &batchv1.CronJob{})
+		cj, ok := obj.(*batchv1.CronJob)
+		if !ok {
+			return nil, fmt.Errorf("type assertion failed: %v", obj)
+		}
+		controller = &cronJobController{
+			obj:         cj,
+			integration: integration,
+			client:      action.client,
+		}
 	default:
-		return fmt.Errorf("unsupported controller for integration %s", integration.Name)
+		return nil, fmt.Errorf("unsupported controller for integration %s", integration.Name)
 	}
 
-	// Retrieve the controller updated from the deployer trait execution
-	controller = environment.Resources.GetController(func(object ctrl.Object) bool {
-		return reflect.TypeOf(controller) == reflect.TypeOf(object)
+	if obj == nil {
+		return nil, fmt.Errorf("unable to retrieve controller for integration %s", integration.Name)
+	}
+
+	return controller, nil
+}
+
+// getUpdatedController retrieves the controller updated from the deployer trait execution.
+func getUpdatedController(env *trait.Environment, obj ctrl.Object) ctrl.Object {
+	return env.Resources.GetController(func(object ctrl.Object) bool {
+		return reflect.TypeOf(obj) == reflect.TypeOf(object)
 	})
-	if controller == nil {
-		return fmt.Errorf("unable to retrieve controller for integration %s", integration.Name)
+}
+
+func (action *monitorAction) updateIntegrationPhaseAndReadyCondition(ctx context.Context, environment *trait.Environment, integration *v1.Integration, pendingPods []corev1.Pod, runningPods []corev1.Pod) error {
+	controller, err := action.newController(environment, integration)
+	if err != nil {
+		return err
 	}
 
-	switch c := controller.(type) {
-	case *appsv1.Deployment:
-		// Check the Deployment progression
-		if progressing := kubernetes.GetDeploymentCondition(*c, appsv1.DeploymentProgressing); progressing != nil && progressing.Status == corev1.ConditionFalse && progressing.Reason == "ProgressDeadlineExceeded" {
-			integration.Status.Phase = v1.IntegrationPhaseError
-			setReadyConditionError(integration, progressing.Message)
-			return nil
-		}
-		podSpec = c.Spec.Template.Spec
+	if done, err := controller.checkReadyCondition(ctx); done || err != nil {
+		return err
+	}
+	if done := checkPodStatuses(integration, pendingPods, runningPods); done {
+		return nil
+	}
+	integration.Status.Phase = v1.IntegrationPhaseRunning
 
-	case *servingv1.Service:
-		// Check the KnativeService conditions
-		if ready := kubernetes.GetKnativeServiceCondition(*c, servingv1.ServiceConditionReady); ready.IsFalse() && ready.GetReason() == "RevisionFailed" {
-			integration.Status.Phase = v1.IntegrationPhaseError
-			setReadyConditionError(integration, ready.Message)
-			return nil
-		}
-		podSpec = c.Spec.Template.Spec.PodSpec
-
-	case *batchv1beta1.CronJob:
-		// Check latest job result
-		if lastScheduleTime := c.Status.LastScheduleTime; lastScheduleTime != nil && len(c.Status.Active) == 0 {
-			jobs := batchv1.JobList{}
-			if err := action.client.List(ctx, &jobs,
-				ctrl.InNamespace(integration.Namespace),
-				ctrl.MatchingLabels{v1.IntegrationLabel: integration.Name},
-			); err != nil {
-				return err
-			}
-			t := lastScheduleTime.Time
-			for i, job := range jobs.Items {
-				if job.Status.Active == 0 && job.CreationTimestamp.Time.Before(t) {
-					continue
-				}
-				lastCompletedJob = &jobs.Items[i]
-				t = lastCompletedJob.CreationTimestamp.Time
-			}
-			if lastCompletedJob != nil {
-				if failed := kubernetes.GetJobCondition(*lastCompletedJob, batchv1.JobFailed); failed != nil && failed.Status == corev1.ConditionTrue {
-					setReadyCondition(integration, corev1.ConditionFalse, v1.IntegrationConditionLastJobFailedReason, fmt.Sprintf("last job %s failed: %s", lastCompletedJob.Name, failed.Message))
-					integration.Status.Phase = v1.IntegrationPhaseError
-					return nil
-				}
-			}
-		}
-		podSpec = c.Spec.JobTemplate.Spec.Template.Spec
+	readyPods, unreadyPods := filterPodsByReadyStatus(runningPods, controller.getPodSpec())
+	if done := controller.updateReadyCondition(readyPods); done {
+		return nil
+	}
+	if err := action.probeReadiness(ctx, environment, integration, unreadyPods); err != nil {
+		return err
 	}
 
+	return nil
+}
+
+func checkPodStatuses(integration *v1.Integration, pendingPods []corev1.Pod, runningPods []corev1.Pod) bool {
 	// Check Pods statuses
 	for _, pod := range pendingPods {
 		// Check the scheduled condition
 		if scheduled := kubernetes.GetPodCondition(pod, corev1.PodScheduled); scheduled != nil && scheduled.Status == corev1.ConditionFalse && scheduled.Reason == "Unschedulable" {
 			integration.Status.Phase = v1.IntegrationPhaseError
 			setReadyConditionError(integration, scheduled.Message)
-			return nil
+			return true
 		}
 	}
 	// Check pending container statuses
@@ -255,7 +271,7 @@ func (action *monitorAction) updateIntegrationPhaseAndReadyCondition(ctx context
 			if waiting := container.State.Waiting; waiting != nil && waiting.Reason == "ImagePullBackOff" {
 				integration.Status.Phase = v1.IntegrationPhaseError
 				setReadyConditionError(integration, waiting.Message)
-				return nil
+				return true
 			}
 		}
 	}
@@ -272,24 +288,26 @@ func (action *monitorAction) updateIntegrationPhaseAndReadyCondition(ctx context
 			if waiting := container.State.Waiting; waiting != nil && waiting.Reason == "CrashLoopBackOff" {
 				integration.Status.Phase = v1.IntegrationPhaseError
 				setReadyConditionError(integration, waiting.Message)
-				return nil
+				return true
 			}
 			if terminated := container.State.Terminated; terminated != nil && terminated.Reason == "Error" {
 				integration.Status.Phase = v1.IntegrationPhaseError
 				setReadyConditionError(integration, terminated.Message)
-				return nil
+				return true
 			}
 		}
 	}
 
-	integration.Status.Phase = v1.IntegrationPhaseRunning
+	return false
+}
 
+func filterPodsByReadyStatus(runningPods []corev1.Pod, podSpec corev1.PodSpec) ([]corev1.Pod, []corev1.Pod) {
 	var readyPods []corev1.Pod
 	var unreadyPods []corev1.Pod
 	for _, pod := range runningPods {
 		// We compare the Integration PodSpec to that of the Pod in order to make
 		// sure we account for up-to-date version.
-		if !equality.Semantic.DeepDerivative(podSpec, pod.Spec) {
+		if !comparePodSpec(podSpec, pod.Spec) {
 			continue
 		}
 		ready := kubernetes.GetPodCondition(pod, corev1.PodReady)
@@ -308,66 +326,53 @@ func (action *monitorAction) updateIntegrationPhaseAndReadyCondition(ctx context
 		}
 	}
 
-	switch c := controller.(type) {
-	case *appsv1.Deployment:
-		replicas := int32(1)
-		if r := integration.Spec.Replicas; r != nil {
-			replicas = *r
-		}
-		// The Deployment status reports updated and ready replicas separately,
-		// so that the number of ready replicas also accounts for older versions.
-		readyReplicas := int32(len(readyPods))
-		switch {
-		case readyReplicas >= replicas:
-			// The Integration is considered ready when the number of replicas
-			// reported to be ready is larger than or equal to the specified number
-			// of replicas. This avoids reporting a falsy readiness condition
-			// when the Integration is being down-scaled.
-			setReadyCondition(integration, corev1.ConditionTrue, v1.IntegrationConditionDeploymentReadyReason, fmt.Sprintf("%d/%d ready replicas", readyReplicas, replicas))
-			return nil
+	return readyPods, unreadyPods
+}
 
-		case c.Status.UpdatedReplicas < replicas:
-			setReadyCondition(integration, corev1.ConditionFalse, v1.IntegrationConditionDeploymentProgressingReason, fmt.Sprintf("%d/%d updated replicas", c.Status.UpdatedReplicas, replicas))
+// comparePodSpec compares given pod spec according to integration specific information (e.g. digest, container image).
+func comparePodSpec(runningPodSpec corev1.PodSpec, referencePod corev1.PodSpec) bool {
+	runningPodContainer := findIntegrationContainer(runningPodSpec)
+	referencePodContainer := findIntegrationContainer(referencePod)
 
-		default:
-			setReadyCondition(integration, corev1.ConditionFalse, v1.IntegrationConditionDeploymentProgressingReason, fmt.Sprintf("%d/%d ready replicas", readyReplicas, replicas))
-		}
+	if runningPodContainer == nil || referencePodContainer == nil {
+		return false
+	}
 
-	case *servingv1.Service:
-		ready := kubernetes.GetKnativeServiceCondition(*c, servingv1.ServiceConditionReady)
-		if ready.IsTrue() {
-			setReadyCondition(integration, corev1.ConditionTrue, v1.IntegrationConditionKnativeServiceReadyReason, "")
-			return nil
-		}
-		setReadyCondition(integration, corev1.ConditionFalse, ready.GetReason(), ready.GetMessage())
+	// integration digest must be the same
+	if getIntegrationDigest(runningPodContainer.Env) != getIntegrationDigest(referencePodContainer.Env) {
+		return false
+	}
 
-	case *batchv1beta1.CronJob:
-		switch {
-		case c.Status.LastScheduleTime == nil:
-			setReadyCondition(integration, corev1.ConditionTrue, v1.IntegrationConditionCronJobCreatedReason, "cronjob created")
-			return nil
+	// integration container image must be the same (same integration kit)
+	if runningPodContainer.Image != referencePodContainer.Image {
+		return false
+	}
 
-		case len(c.Status.Active) > 0:
-			setReadyCondition(integration, corev1.ConditionTrue, v1.IntegrationConditionCronJobActiveReason, "cronjob active")
-			return nil
+	return true
+}
 
-		case c.Spec.SuccessfulJobsHistoryLimit != nil && *c.Spec.SuccessfulJobsHistoryLimit == 0 && c.Spec.FailedJobsHistoryLimit != nil && *c.Spec.FailedJobsHistoryLimit == 0:
-			setReadyCondition(integration, corev1.ConditionTrue, v1.IntegrationConditionCronJobCreatedReason, "no jobs history available")
-			return nil
-
-		case lastCompletedJob != nil:
-			if complete := kubernetes.GetJobCondition(*lastCompletedJob, batchv1.JobComplete); complete != nil && complete.Status == corev1.ConditionTrue {
-				setReadyCondition(integration, corev1.ConditionTrue, v1.IntegrationConditionLastJobSucceededReason, fmt.Sprintf("last job %s completed successfully", lastCompletedJob.Name))
-				return nil
-			}
-
-		default:
-			integration.Status.SetCondition(v1.IntegrationConditionReady, corev1.ConditionUnknown, "", "")
+func getIntegrationDigest(envs []corev1.EnvVar) string {
+	for _, env := range envs {
+		if env.Name == digest.IntegrationDigestEnvVar {
+			return env.Value
 		}
 	}
 
-	// Finally, call the readiness probes of the non-ready Pods directly,
-	// to retrieve insights from the Camel runtime.
+	return ""
+}
+
+func findIntegrationContainer(spec corev1.PodSpec) *corev1.Container {
+	for _, c := range spec.Containers {
+		if c.Name == "integration" {
+			return &c
+		}
+	}
+
+	return nil
+}
+
+// probeReadiness calls the readiness probes of the non-ready Pods directly to retrieve insights from the Camel runtime.
+func (action *monitorAction) probeReadiness(ctx context.Context, environment *trait.Environment, integration *v1.Integration, unreadyPods []corev1.Pod) error {
 	var runtimeNotReadyMessages []string
 	for i := range unreadyPods {
 		pod := &unreadyPods[i]
@@ -391,15 +396,11 @@ func (action *monitorAction) updateIntegrationPhaseAndReadyCondition(ctx context
 				runtimeNotReadyMessages = append(runtimeNotReadyMessages, fmt.Sprintf("readiness probe failed for Pod %s/%s: %s", pod.Namespace, pod.Name, err.Error()))
 				continue
 			}
-			health := HealthCheck{}
-			err = json.Unmarshal(body, &health)
+			health, err := NewHealthCheck(body)
 			if err != nil {
 				return err
 			}
 			for _, check := range health.Checks {
-				if check.Name != "camel-readiness-checks" {
-					continue
-				}
 				if check.Status == HealthCheckStateUp {
 					continue
 				}
