@@ -143,7 +143,7 @@ func (r *reconcileKamelet) Reconcile(ctx context.Context, request reconcile.Requ
 	}
 
 	// Only process resources assigned to the operator
-	if !platform.IsOperatorHandler(&instance) {
+	if !platform.IsOperatorHandlerConsideringLock(ctx, r.client, request.Namespace, &instance) {
 		rlog.Info("Ignoring request because resource is not assigned to current operator")
 		return reconcile.Result{}, nil
 	}
@@ -178,6 +178,8 @@ func (r *reconcileKamelet) Reconcile(ctx context.Context, request reconcile.Requ
 		}
 
 		if target != nil {
+			target.Status.ObservedGeneration = instance.GetGeneration()
+
 			if err := r.client.Status().Patch(ctx, target, ctrl.MergeFrom(&instance)); err != nil {
 				camelevent.NotifyKameletError(ctx, r.client, r.recorder, &instance, target, err)
 				return reconcile.Result{}, err
