@@ -20,6 +20,7 @@ package resources
 import (
 	"bytes"
 	"io/ioutil"
+	"net/http"
 	"os"
 	"path/filepath"
 	"strings"
@@ -46,7 +47,7 @@ func Resource(name string) ([]byte, error) {
 		name = "/" + name
 	}
 
-	file, err := assets.Open(name)
+	file, err := openAsset(name)
 	if err != nil {
 		return nil, errors.Wrapf(err, "cannot access resource file %s", name)
 	}
@@ -85,7 +86,7 @@ func TemplateResource(name string, params interface{}) (string, error) {
 
 // DirExists tells if a directory exists and can be listed for files.
 func DirExists(dirName string) bool {
-	if _, err := assets.Open(dirName); err != nil {
+	if _, err := openAsset(dirName); err != nil {
 		return false
 	}
 	return true
@@ -103,8 +104,9 @@ func WithPrefix(pathPrefix string) ([]string, error) {
 
 	var res []string
 	for i := range paths {
-		if result, _ := filepath.Match(pathPrefix+"*", paths[i]); result {
-			res = append(res, paths[i])
+		path := filepath.ToSlash(paths[i])
+		if result, _ := filepath.Match(pathPrefix+"*", path); result {
+			res = append(res, path)
 		}
 	}
 
@@ -113,7 +115,7 @@ func WithPrefix(pathPrefix string) ([]string, error) {
 
 // Resources lists all file names in the given path (starts with '/').
 func Resources(dirName string) ([]string, error) {
-	dir, err := assets.Open(dirName)
+	dir, err := openAsset(dirName)
 	if err != nil {
 		if os.IsNotExist(err) {
 			return nil, nil
@@ -146,3 +148,7 @@ func Resources(dirName string) ([]string, error) {
 
 	return res, dir.Close()
 }
+
+func openAsset(path string) (http.File, error) {
+	return assets.Open(filepath.ToSlash(path))
+}
